//
//                                                                         
// Module: Internals Example Source File                                   
//                                                                         
// Description: Declarations for the Internals Example Plugin              
//                                                                         
//                                                                         
// This source code module, and all information, data, and algorithms      
// associated with it, are part of CUBE technology (tm).                   
//                 PROPRIETARY AND CONFIDENTIAL                            
// Copyright (c) 1996-2008 Image Space Incorporated.  All rights reserved. 
//                                                                         
//                                                                         
// Change history:                                                         
//   tag.2005.11.30: created                                               
//                                                                         
//

#define _CRT_SECURE_NO_WARNINGS 1

#include "Example.hpp"          // corresponding header file
#include <math.h>               // for atan2, sqrt
#include <stdio.h>
#include <string>
#include <iostream>



#pragma comment(lib, "Ws2_32.lib")

	SOCKET s;
	SOCKADDR_IN brdcastaddr;
	

// plugin information

extern "C" __declspec( dllexport )
const char * __cdecl GetPluginName()                   { return( "ExamplePlugin - 2008.02.13" ); }

extern "C" __declspec( dllexport )
PluginObjectType __cdecl GetPluginType()               { return( PO_INTERNALS ); }

extern "C" __declspec( dllexport )
int __cdecl GetPluginVersion()                         { return( 1 ); } // InternalsPluginV01 functionality

extern "C" __declspec( dllexport )
PluginObject * __cdecl CreatePluginObject()            { return( (PluginObject *) new ExampleInternalsPlugin ); }

extern "C" __declspec( dllexport )
void __cdecl DestroyPluginObject( PluginObject *obj )  { delete( (ExampleInternalsPlugin *) obj ); }


// ExampleInternalsPlugin class

//void ExampleInternalsPlugin::WriteToAllExampleOutputFiles( const char * const openStr, const char * const msg )
//{
//}


void ExampleInternalsPlugin::Startup( long version )
{
  	int portno = 12345;
    WORD w = MAKEWORD(1,1);
    WSADATA wsadata;
    ::WSAStartup(w, &wsadata);
	
	s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    
    char opt = 1; 
    setsockopt(s, SOL_SOCKET, SO_BROADCAST, (char*)&opt, sizeof(char));
    memset(&brdcastaddr,0, sizeof(brdcastaddr));
    brdcastaddr.sin_family = AF_INET;
    brdcastaddr.sin_port = htons(portno);
    brdcastaddr.sin_addr.s_addr = INADDR_BROADCAST;
}

void ExampleInternalsPlugin::UpdateTelemetry( const TelemInfoV01 &info )
{
    char buffer [200];
	int len = sizeof(brdcastaddr);

	 // Compute some auxiliary info based on the above
    TelemVect3 forwardVector = { -info.mOri[0].z, -info.mOri[1].z, -info.mOri[2].z };
	TelemVect3    upVector = {  info.mOri[0].y,  info.mOri[1].y,  info.mOri[2].y };
    TelemVect3    leftVector = {  info.mOri[0].x,  info.mOri[1].x,  info.mOri[2].x };

    // These are normalized vectors, and remember that our world Y coordinate is up.  So you can
    // determine the current pitch and roll (w.r.t. the world x-z plane) as follows:
    const double pitch = atan2( forwardVector.y, sqrt( ( forwardVector.x * forwardVector.x ) + ( forwardVector.z * forwardVector.z ) ) );
	const double yaw = atan2(info.mOri[0].z, info.mOri[2].z) ;
    const double  roll = atan2(    leftVector.y, sqrt( (    leftVector.x *    leftVector.x ) + (    leftVector.z *    leftVector.z ) ) );
    const double radsToDeg = 57.296;

	//sprintf(buffer, "%.3f, %.3f, %.3f, %.3f, %.3f, %.3f\r\n", info.mLocalRotAccel.z, info.mLocalRotAccel.x, info.mLocalAccel.y, info.mLocalRotAccel.y, - info.mLocalAccel.x, - info.mLocalAccel.z);
	//sprintf(buffer, "%.3f, %.3f, %.3f, %.3f, %.3f, %.3f\r\n", info.mLocalRot.z, info.mLocalRot.x, info.mLocalAccel.y, info.mLocalRot.y, - info.mLocalAccel.x, - info.mLocalAccel.z);
	//sprintf(buffer, "%.3f, %.3f, %.3f, %.3f, %.3f, %.3f\r\n", roll * radsToDeg,  pitch * radsToDeg, info.mLocalAccel.y, yaw * radsToDeg, - info.mLocalAccel.x, - info.mLocalAccel.z);
	sprintf(buffer, "0, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, 0, 0, 0, %ld, %ld, %.3f, %.3f, 0, %.3f, %.3f, 0, %.3f, 0, 0\r\n", info.mLocalRotAccel.z, - info.mLocalRotAccel.x, info.mLocalRotAccel.y, yaw, pitch, roll, - info.mLocalAccel.z, - info.mLocalAccel.x, info.mLocalAccel.y, - info.mLocalVel.z, - info.mLocalVel.x, info.mLocalVel.y, - info.mPos.z, - info.mPos.x, info.mPos.y, info.mGear, info.mID, - info.mLocalVel.z, info.mEngineRPM, info.mEngineWaterTemp, info.mFuel, info.mEngineOilTemp);
	//                                                                                                                                                                              RotAccZ,               RotAccX,                 RotAccY,               yaw, pitch, roll, SurgeAcc,             SwayAcc,              HeaveAcc,           forward velocity,   right velocity,     up velocity,                                                 currentgear,playerID, speed,              engineRPM,       engine temp,           fuel,       oil temp
    int ret = sendto(s, buffer, strlen(buffer), 0, (sockaddr*)&brdcastaddr, len);
}

void ExampleInternalsPlugin::Shutdown()
{
    ::closesocket(s);
}


void ExampleInternalsPlugin::StartSession()
{
}


void ExampleInternalsPlugin::EndSession()
{
}


void ExampleInternalsPlugin::EnterRealtime()
{
}


void ExampleInternalsPlugin::ExitRealtime()
{
}


bool ExampleInternalsPlugin::CheckHWControl( const char * const controlName, float &fRetVal )
{
  return( false );
}


bool ExampleInternalsPlugin::ForceFeedback( float &forceValue )
{
  return( false );
}


void ExampleInternalsPlugin::UpdateGraphics( const GraphicsInfoV01 &info )
{
}


void ExampleInternalsPlugin::UpdateScoring( const ScoringInfoV01 &info )
{
}


bool ExampleInternalsPlugin::RequestCommentary( CommentaryRequestInfoV01 &info )
{
  return( false );
}

